<?php
 /**
   * Name: Expenditure.php
   * Author: R. Joseph Newton
   * Purpose: Provides Expenditure class for Project Manager
   *           applications
   * Class provided: Expenditure
   * Last modified: February 3, 2007
   */

include_once('project_defs.php');
include_once('MySQLAdapterProjectManager.php');
include_once('Person.php');

class Expenditure {
   private $_expenditure_guid;
   private $_project_guid;
   private $_authorized_by;
   private $_amount;
   private $_date_incurred;
   private $_payee;
   private $_title;
   private $_description;
   private $_registered;
   private $_db_adapter;
   private $_modified;
   
   public function Expenditure($project_guid, $authorized_by, $amount, array $date_incurred,
    $payee, $title, $description = '', $registered) {
      $this->_project_guid = $project_guid;
      $this->_authorized_by = $authorized_by;
      $this->_amount = $amount;
      $this->_date_incurred = $date_incurred;
      $this->_payee = $payee;
      $this->_title = $title;
      $this->_description = $description;
      $this->_registered = $registered;
      $this->_db_adapter = new MySQLAdapterProjectManager();
      $this->modified = true;
   }
   
   public function load_from_array(array $array) {
      $date_incurred = date_ISO_to_array($array['date_incurred']);
      $expenditure = new Expenditure($array['project_guid'], $array['authorized_by'],
       $array['amount'], $date_incurred, $array['payee'], $array['title'],
       $array['description'], $array['registered']);
      $expenditure->_expenditure_guid = $array['expenditure_guid'];
      $expenditure->_modified = false;
      return $expenditure;
   }
   
   public function get_by_expenditure_id($expenditure_id) {
      $adapter = new MySQLAdapterProjectManager();
      $expenditure = NULL;
      $query = "select * from expenditure where expenditure_guid = $expenditure_id";
      $result = $adapter->db_get_first($query);
      $date_incurred = date_ISO_to_array($result['date_incurred']);
      $registered = date_ISO_to_array($result['registered']);
      $description = isset($result['description']) ? $result['description'] : '';
      $expenditure = new Expenditure($result['project_guid'], $result['authorized_by'],
       $result['amount'], $date_incurred, $result['payee'],  $result['title'],
       $description, $registered);
      $expenditure->_expenditure_guid = $expenditure_id;
      $expenditure->_modified = false;
      return $expenditure;
   }
   
   public function save() {
      if ( isset($this->_expenditure_guid) ) {
         if ($this->_modified) {
            $result = $this->_update();
         } else {
            $result = true;
         }
      } else {
         $result = $this->_create_new();
      }
      if ($result == 1) {
         $this->__modified = false;
      }
      return $result;
   }
   
   private function _update() {
      $adapter = $this->_db_adapter;
      $this->_amend();
      $date_incurred = date_array_to_ISO($this->_date_incurred);
      $payee = $adapter->db_escape($this->_payee);
      $title = $adapter->db_escape($this->_title);
      $description = $adapter->db_escape($this->_description);
      $command = "update expenditure set" .
      " project_guid = $this->_project_guid," .
      " authorized_by = $this->_authorized_by," .
      " amount = $this->_amount," .
      " date_incurred = $date_incurred," .
      " payee = '$payee'," .
      " title = '$title'," .
      " description = '$description'" .
      "    where expenditure_guid = $this->_expenditure_guid";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   private function _amend() {
      $adapter = $this->_db_adapter;
      $change_guid = $adapter->create_table_guid('expenditure_change');
      $date_now = date('Y-m-d');
      $command = "insert into expenditure_change select * from" .
      " (select $change_guid, '$date_now') as change_data join" .
      " (select * from expenditure" .
      " where expenditure_guid = $this->_expenditure_guid) as existing_row;";
      $result = $adapter->db_command($command);
   }
   
   private function _create_new() {
      $adapter = $this->_db_adapter;
      $guid = $adapter->create_table_guid('expenditure');
      if (!$guid) {
         return false;
      }
      $this->_expenditure_guid = $guid;
      $command = $this->_insert_expenditure_command();
      print "Command is \"$command\"<br>\n";
      $result = $adapter->db_command($command);
      if ( is_array($result) ) {
         dump_array_to_page($result);
      }
      return $result;
   }
   
   private function _insert_expenditure_command() {
      $adapter = $this->_db_adapter;
      $date_incurred = date_array_to_ISO($this->_date_incurred);
      $payee = $adapter->db_escape($this->_payee);
      $title = $adapter->db_escape($this->_title);
      $registered = Date('Y-m-d');
      $description = $this->_description ? $adapter->db_escape($this->_description) : '';
      $command = "insert into expenditure values ($this->_expenditure_guid," .
      " $this->_project_guid, $this->_authorized_by, $this->_amount, $date_incurred, '$payee'," .
      " '$title', '$description', '$registered')";
      return $command;
   }
   
   public function get_expenditure_id() {
      return $this->_expenditure_guid;
   }
   
   public function get_project_id() {
      return $this->_project_guid;
   }
   
   public function change_project_designation($project_id) {
      $original = $this->_project_guid;
      $this->_project_guid = $project_id;
      if ($original != $project_id) {
         $this->_modified = true;
      }
   }
   
   public function get_authorized_by() {
      return $this->_authorized_by;
   }

   public function amend_authorized_by($new_authorization) {
      $original = $this->_authorized_by;
      $this->_authorized_by = $new_authorization;
      if ($original != $new_authorization) {
         $this->_modified = true;
      }
   }
   
   public function get_expenditure_amount() {
      return $this->_amount;
   }
   
   public function adjust_amount($new_amount) {
      $original = $this->_amount;
      $this->_amount = $new_amount;
      if ($original != $new_amount) {
         $this->_modified = true;
      }
   }
   
   public function get_date_incurred() {
      return $this->_date_incurred;
   }
   
   public function amend_date_incurred($adjusted_date) {
      $original = $this->_date_incurred;
      $this->_date_incurred = $adjusted_date;
      if ($original != $adjusted_date) {
         $this->_modified = true;
      }
   }
   
   public function get_payee() {
      return $this->_payee;
   }
   
   public function change_payee($new_payee) {
      $original = $this->_payee;
      $this->_payee = $new_payee;
      if ($original != $new_payee) {
         $this->_modified = true;
      }
   }
   
   public function get_line_item_title() {
      return $this->_title;
   }
   
   public function amend_line_item_title($new_title) {
      $original = $this->_title;
      $this->_title = $new_title;
      if ($original != $new_title) {
         $this->_modified = true;
      }
   }
   
   public function get_expenditure_description() {
      return $this->_description;
   }
   
   public function modify_expenditure_description($new_description) {
      $original = $this->_description;
      $this->_description = $new_description;
      if ($original != $new_description) {
         $this->_modified = true;
      }
   }
   
   public function is_modified() {
      return $this->_modified;
   }
   
   public function get_original_registry_date() {
      return $this->_registered;
   }
   
   public function as_array() {
      $expenditure_array = array('expenditure_id' => $this->_expenditure_guid,
       'project_id' => $this->_project_guid, 'authorized_by' => $this->_authorized_by,
       'amount' => $this->_amount, 'payee' => $this->_payee, 'title' => $this->_title);
      if ( isset($this->_description) ) {
         $expenditure_array[] = array('description' => $this->_description);
      }
   }
   
   public function to_string() {
      $expenditure_string = "$this->_expenditure_guid: $this->_title\n" .
       "Paid to $this->_payee:              $this->_amount\n" .
       "Autorized by: $this->_authorized_by\n";
      if ( isset($this->_description) ) {
         $expenditure_string .= "Description:\n$this->_description\n";
      }
   }
}

?>
