<?php
 /**
  * Name: Milestone.php
  * Author: R. Joseph Newton
  * Purpose: Provides Milestone class for Project Manager applications
  * Class provided: Milestone
  * Last modified: February 3, 2007
  */

include_once('project_defs.php');
include_once('MySQLAdapterProjectManager.php');

class Milestone {
   private $_milestone_guid;
   private $_project_guid;
   private $_title;
   private $_significance;
   private $_description;
   private $_date_goal;
   private $_date_achieved;
   private $_registered;
   
   private $_db_adapter;
   private $_modified;
   
   public function Milestone($project_id, $title, $significance, $description,
    $goal_date = NULL, $date_achieved = NULL, $registered) {
      $this->_project_guid = $project_id;
      $this->_title = $title;
      $this->_significance = $significance;
      $this->_description = $description;
      if ( isset($goal_date) ) {
         $this->_date_goal = $goal_date;
      }
      if ( isset($date_achieved) ) {
         $this->_date_achieved = $date_achieved;
      }
      $this->_registered = $registered;
      $this->_db_adapter = new MySQLAdapterProjectManager();
      $this->_modified = true;
   }
   
   public function load_from_array(array $array) {
      $milestone = new Milestone($array['project_guid'], $array['title'],
       $array['significance'], $array['description'], $array['date_goal'],
       $array['date_achieved'], $array['registered']);
      $milestone->_milestone_guid = $array['milestone_guid'];
      $milestone->_modified = false;
      return $milestone;
   }
   
   public function get_by_milestone_id($milestone_id) {
      $adapter = new MySQLAdapterProjectManager();
      $allocation = NULL;
      $query = "select * from milestone where milestone_guid = $milestone_id";
      $result = $adapter->db_get_first($query);
      $date_goal = date_ISO_to_array($result['date_goal']);
      $date_achieved = date_ISO_to_array($result['date_achieved']);
      $milestone = new Milestone($result['project_guid'], $result['title'],
       $result['significance'], $result['description'], $date_goal, $date_achieved,
       $result['registered']);
      $milestone->_milestone_guid = $milestone_id;
      $milestone->_modified = false;
      return $milestone;
   }
   
   public function save() {
      if ( isset($this->_milestone_guid) ) {
         $result = $this->_update();
      } else {
         $result = $this->_create_new();
      }
      if ($result == 1) {
         $this->__modified = false;
      }
   }
   
   private function _update() {
      $adapter = $this->_db_adapter;
      $title = $adapter->db_escape($this->_title);
      $significance = $adapter->db_escape($this->_significance);
      $description = $adapter->db_escape($this->_description);
      $date_goal = date_array_to_ISO($this->_date_goal);
      $date_achieved = date_array_to_ISO($this->_date_achieved);
      $command = "update milestone set project_guid = $this->_project_guid," .
      "title = '$title', significance = '$significance', description = '$description'," .
      " date_goal = $date_goal, date_achieved = $date_achieved";
      $command .= " where milestone_guid = $this->_milestone_guid";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   private function _create_new() {
      $adapter = $this->_db_adapter;
      $guid = $adapter->create_table_guid('milestone');
      if (!$guid) {
         return false;
      }
      $this->_milestone_guid = $guid;
      $query = $this->_insert_milestone_query();
      $result = $adapter->db_command($query);
      return $result;
   }
   
   private function _insert_milestone_query() {
      $adapter = $this->_db_adapter;
      $title = $adapter->db_escape($this->_title);
      $significance = $adapter->db_escape($this->_significance);
      $description = $adapter->db_escape($this->_description);
      $date_goal = isset($this->_date_goal) ?
       date_array_to_ISO($this->_date_goal) : 'NULL';
      $date_achieved = isset($this->_date_achieved) ?
       date_array_to_ISO($this->_date_achieved) : 'NULL';
      $query = "insert into milestone values ($this->_milestone_guid," .
      " $this->_project_guid, '$title', '$significance', '$description', $date_goal," .
      " $date_achieved, '$this->_registered')";
      return $query;
   }
   
   public function get_milestone_id() {
      return $this->_milestone_guid;
   }
   
   public function get_project_id() {
      return $this->_project_guid;
   }
   
   public function change_project_designation($new_project_id) {
      $this->_project_guid = $new_project_id;
      $this->_modified = true;
   }
   
   public function get_title() {
      return $this->_title;
   }
   
   public function change_title($new_title) {
      $this->_title = $new_title;
      $this->_modified = true;
   }
   
   public function get_significance() {
      return $this->_significance;
   }
   
   public function re_express_significance($significance) {
      $this->_significance = $significance;
      $this->_modified = true;
   }
   
   public function get_description() {
      return $this->_description;
   }
   
   public function modify_milestone_description($description) {
      $this->_description = $description;
      $this->_modified = true;
   }
   
   public function get_target_date() {
      return $this->_date_goal;
   }
   
   public function adjust_target_date($target_date) {
      $this->_date_goal = $target_date;
      $this->_modified = true;
   }
   
   public function get_date_achieved() {
      return $this->_date_achieved;
   }
   
   public function record_date_achieved($date_achieved) {
      $this->_date_achieved = $date_achieved;
      $this->_modified = true;
   }
   
   public function get_original_registry_date() {
      return $this->_registered;
   }
   
   public function as_array() {
      $milestone_array = array('milestone_id' => $this->_milestone_guid,
       'project_id' => $this->_project_guid, 'title' => $this->_title,
       'significance' => $this->_significance, 'description' => $this->_description);
      if ( isset($this->_date_goal) ) {
         $allocation_array[] = array('date_goal' => $this->_date_goal);
      }
      if ( isset($this->_date_achieved) ) {
         $allocation_array[] = array('date_achieved' => $this->_date_achieved);
      }
      return $allocation_array;
   }
   
   public function to_string() {
      $date_goal = isset($this->_date_achieved) ?
       date_array_to_ISO($this->_date_goal) : '';
      $date_achieved = isset($this->_date_achieved) ?
       date_array_to_ISO($this->_date_achieved) : '';
      $milestone_string = "$this->_milestone_guid: $this->_title\n" .
       "Project ID: $this->_project_guid\n" .
       "Significance: $this->_significance\n";
      if ( isset($date_goal) ) {
         $milestone_string .= "Target date: $date_goal\n";
      }
      if ( isset($this->_description) ) {
         $milestone_string .= "Description:\n$this->_description\n";
      }
      if ( isset($date_achieved) ) {
         $milestone_string .= "Date achieved: $date_achieved\n";
      }
      return $milestone_string;
   }
}

?>
