<?php
 /**
  * Name: Person.php
  * Author: R. Joseph Newton
  * Class provided: Person
  * Purpose: Provides a Person class for use in management applications
  * Last modified: February 3, 2007
  */

 /**
  * @Methods:
  *  @constructor Person (string given_name, string last_name, enum preferred_mode [, string first_name
  *   [, string middle_name [, array birth_date]]])
  *   @Returns a new Person object with mandatory attributes set. [Note: DO NOT call directly for an
  *     exiting Person object.  Use get_by_person_id() as appropriate to load GUID.  Call save() on new
  *     Person objects to load GUID.]
  *   @Params
  *    @ preferred_mode -- can be either 'telephone' or 'e-mail'.
  *    @ birth_date -- an associative array with keys: 'year', 'month', 'day', and numerical values.
  *      [year must be full four-digit number]
  *      
  *  @Person get_by_person_id(int person_guid)
  *   @Returns an existing Person object, reference by person_guid.
  *
  *  @mixed save ()
  *   @Action: Saves the existing Person object.  This method will either create anew record
  *    or update the existing one, as appropriate.
  *
  *  @int get_person_guid ()
  *   @Returns the GUID for the current person object.
  *
  *  @array get_person_name ()
  *   @Returns an associative array with keys 'last, 'given', 'first', middle'.
  *
  *  @void set_person_name (array name)
  *   @Action: Sets the name of the current person object
  *   @Param:
  *     name -- an associative array with mandatory keys 'last', 'given'
  *      and optional keys 'first', 'middle'.
  *
  *  @array get_birth_date ()
  *   @Returns an associative array with keys 'year, 'month', 'day' if set, an empty array
  *     otherwise.  Test with isset($returned_var['year']) before attempting to acces elements.
  *
  *  @void set_birth_date (array date)
  *   @Action: Sets birth_date.
  *   @Param:
  *     birth_date -- an associative array with keys 'year', 'month', 'day', and numerical values.
  *      [year must be full four-digit number]
  *
  *  @string get_primary_e_mail ()
  *   @Returns the the current Person's designated primary e-mail address.
  *
  *  @mixed add_e_mail (string e_mail [, boolean primary])
  *   @Action: Adds a new e-mail address for the current Person to the record.
  *   @Returns either the number 1 on success, or an associative array with elements
  *     'return_code' => [number] and 'error' => [database error message]
  *   @Params:
  *     @e-mail -- the e-mail address being added
  *     @primary -- flag to indicate that this is the primary e-mail address to use in contacting
  *      the person
  *
  *  @int remove_e_mail (string e_mail)
  *  @array get_primary_phone_number ()
  *  @array add_phone_number (string phone_number [, boolean primary])
  *  @int remove_phone_number (string phone_number)
  *  @array as_array ()
  *  @string to_string ()
  */

include_once('project_defs.php');
include_once('utilities.php');
include_once('MySQLAdapterProjectManager.php');

class Person {
   private $_person_guid;
   private $_name_given;
   private $_name_last;
   private $_preferred_mode;
   private $_name_first;
   private $_name_middle;
   private $_birth_date;
   private $_db_adapter;
   private $_modified;

   public function Person($given_name, $last_name, $preferred_mode, $first_name = '',
    $middle_name = '', $birth_date = array() ) {
      $this->_name_given = $given_name;
      $this->_name_last = $last_name;
      $this->_preferred_mode = $preferred_mode;
      $this->_name_first = $first_name;
      $this->_name_middle = $middle_name;
      $this->_birth_date = $birth_date;
      $this->_db_adapter = new MySQLAdapterProjectManager();
      $this->modified = true;
   }
   
   public function get_by_person_id($person_guid) {
      $adapter = new MySQLAdapterProjectManager();
      $query = "select * from person where person_guid = $person_guid";
      $result = $adapter->db_get_first($query);
      $person = NULL;
      $birth_date = date_ISO_to_array($result['birth_date']);
      $person = new Person($result['name_given'], $result['name_last'],
       $result['preferred_mode'], $result['name_first'], $result['name_middle'], $birth_date);
      $person->_person_guid = $person_guid;
      return $person;
   }
   
   public function save() {
      if ( isset($this->_person_guid) ) {
         $result = $this->_update();
      } else {
         $result = $this->_create_new();
      }
      if ($result == 1) {
         $this->_modified = false;
      }
      return $result;
   }
   
   private function _create_new() {
      $adapter = $this->_db_adapter;
      $guid = $adapter->create_table_guid('person');
      if (!$guid) {
         return false;
      }
      $this->_person_guid = $guid;
      $insert_query = $this->_insert_person_query();
      $result = $adapter->db_command($insert_query);
      return $result;
   }
   
   private function _create_person_guid() {
      $adapter = $this->_db_adapter;
      $result_array = $adapter->get_procedure_results('increment_person_guid', array('@guid' => 'OUT'));
      $guid = $result_array['@guid'];
      $this->_person_guid = $guid;
      return $guid;
   }
   
   private function _insert_person_query() {
      $adapter = $this->_db_adapter;
      $query = 'insert into person values (';
      $birth_date = $this->_birth_date;
      $birth_date_field = date_array_to_ISO($birth_date);
      if (isset($birth_date_field)) {
         $birth_date_field ="'$birth_date_field'";
      } else {
         $birth_date_field = 'null';
      }
      $name_last = $adapter->db_escape($this->_name_last);
      $name_given = $adapter->db_escape($this->_name_given);
      $name_first = $adapter->db_escape($this->_name_first);
      $name_middle = $adapter->db_escape($this->_name_middle);
      $query .= "$this->_person_guid, '$name_last', '$this->_name_given'," .
      " '$this->_preferred_mode', '$name_first', '$name_middle', $birth_date_field)";
      return $query;
   }
   
   private function _update() {
      $adapter = $this->_db_adapter;
      $birth_date_field = date_array_to_ISO($birth_date);
      $name_last = $adapter->db_escape($this->_name_last);
      $name_given = $adapter->db_escape($this->_name_given);
      $name_first = $adapter->db_escape($this->_name_first);
      $name_middle = $adapter->db_escape($this->_name_middle);
      $command = "update person set name_last = '$name_last', name_given = '$name_given'," .
      " preferred_mode = '$this->_preferred_mode', name_first = '$name_first'," .
      " name_middle = '$name_middle', $birth_date_field" .
      " where person_guid = $this->_person_guid";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   public function get_person_id() {
      return $this->_person_guid;
   }
   
   public function get_person_name_array() {
      return array (
         'last' => $this->_name_last,
         'given' => $this->_name_given,
         'first' => $this->_name_first,
         'middle' => $this->_name_middle
      );
   }
   
   public function get_daily_name() {
      return $this->_name_given . ' ' . $this->_name_last;
   }
   
   public function set_person_name(array $name) {
      $this->_name_last = $name['last'];
      $this->_name_given = $name['given'];
      if ( isset($name['first']) ) {
         $this->_name_first = $name['first'];
      }
      if ( isset($name['middle']) ) {
         $this->_name_middle = $name['middle'];
      }
      $this->modified = true;
   }
   
   public function get_given_name() {
      return $this->_name_given;
   }
   
   public function set_given_name($given_name) {
      if (!$given_name) {
         return;
      }
      $this->_name_given = $given_name;
   }
   
   public function get_last_name() {
      return $this->_name_last;
   }
   
   public function set_last_name($last_name) {
      if (!$last_name) {
         return;
      }
      $this->_name_last = $last_name;
   }
   
   public function get_first_name() {
      return $this->_name_first;
   }
   
   public function set_first_name($first_name) {
      $this->_name_first = $first_name;
   }
   
   public function get_middle_name() {
      return $this->_name_middle;
   }
   
   public function set_middle_name($middle_name) {
      $this->_name_middle = $middle_name;
   }
   
   public function get_birth_date() {
      return $this->_birth_date;
   }
   
   public function set_birth_date(array $birth_date) {
      $this->_birth_date = $birth_date;
      $this->modified = true;
   }
   
   public function get_primary_e_mail() {
      $adapter = $this->_db_adapter;
      $query = 'select * from e_mail where' .
      " person_guid = $this->_person_guid and is_primary = true";
      $result = db_get_first($query);
      return $result['e-mail'];
   }
   
   public function add_e_mail($e_mail, $primary = false) {
      /* TODO: Add code ensuring that there is always one and only one primary addy for each person */
      $adapter = $this->_db_adapter;
      $e_mail = $adapter->db_escape($e_mail);
      $command = "insert into e_mail values ($this->_person_guid, '$e_mail', $primary)";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   public function remove_e_mail($e_mail) {
      /* TODO: Add code ensuring that there is always one and only one primary addy for each person */
      $adapter = $this->_db_adapter;
      $e_mail = $adapter->db_escape($e_mail);
      $command = "delete from e_mail where person_guid = $this->_person_guid and e_mail = $e_mail";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   public function get_primary_phone_number() {
      $adapter = $this->_db_adapter;
      $query = 'select * from phone_number where' .
      " person_guid = $this->_person_guid and is_primary = true";
      $result = db_get_first($query);
      return $result;
   }
   
   public function add_phone_number($phone_number, $primary = false) {
      /* TODO: Add code ensuring that there is always one and only one primary number for each person */
      $adapter = $this->_db_adapter;
      $command = "insert into phone_number values ($this->_person_guid, '$phone_number', $primary)";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   public function remove_phone_number($phone_number) {
      /* TODO: Add code ensuring that there is always one and only one primary number for each person */
      $adapter = $this->_db_adapter;
      $command = "delete from phone_number where person_guid = $this->_person_guid and phone_number = $phone_number";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   public function is_modified() {
      return $this->_modified;
   }
   
   public function as_array() {
      $person_array = array('name_given' => $this->_name_given, 'name_last' => $this->_name_last,
       'preferred_mode' => $this->_preferred_mode);
      if ( isset($this->_name_first) ) {
         $person_array[] = array('name_first' => $this->_name_first);
      }
      if ( isset($this->_name_middle) ) {
         $person_array[] = array('name_middle' => $this->_name_middle);
      }
      if ( isset($this->_birth_date['year']) ) {
         $person_array[] = array('birth_date' => $this->_birth_date);
      }
      return $person_array;
   }
   
   public function to_string() {
      $string = "$this->_person_guid: $this->_name_given $this->_name_last\n";
      $string .= "First:    $this->_name_first\n";
      $string .= "Middle:   $this->_name_middle\n";
      $string .= "Prefers:  $this->_preferred_mode\n";
      if ( isset($this->_birth_date['year']) ) {
         $string .= 'Born: ' . $this->_birth_date['year'] . '-' . $this->_birth_date['month'] . '-' . $this->_birth_date['day'] . "\n";
      }
      return $string;
   }
   
   function person_sort_given_first(Person $person1, Person $person2) {
      $given_name1 = $person1->get_given_name();
      $given_name2 = $person2->get_given_name();
      $last_name1 = $person1->get_last_name();
      $last_name2 = $person2->get_last_name();
      if ($given_name1 > $given_name2) {
         return 1;
      } elseif ($given_name1 < $given_name2) {
         return -1;
      } else {
         if ($last_name1 > $last_name2) {
            return 1;
         } elseif ($last_name1 < $last_name2) {
            return -1;
         }
      }
      return 0;
   }
}
?>
